<?php
/**
 * Copyright 2008 Center for History and New Media
 *
 * This file is part of OAI-PMH Parser Library.
 *
 * OAI-PMH Parser Library is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.

 * OAI-PMH Parser Library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with OAI-PMH Parser Library.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

abstract class OaiPmh {

  const NAMESPACE_URI = 'http://www.openarchives.org/OAI/2.0/';

  /**
   * Factory method to build the request, response, and DOM objects. It 
   * returns the parsed DOM object only.
   * @param $requestVerb
   * @param $baseUrl
   * @param $requestArguments
   * @return unknown_type
   */
  public static function factory($requestVerb, $baseUrl, $requestArguments = array()) {
    $request = new OaiPmh_Request($requestVerb, $baseUrl);
    $request->addRequestArguments($requestArguments);
    $response = $request->getResponseObject();
    $dom = $response->getDomObject();
    return $dom->getOaiPmh();
  }

  /**
   * Validate the request verb.
   * @param $requestVerb
   * @return unknown_type
   */
  protected static function validateRequestVerb($requestVerb) {
    $requestVerbs = array(
      'Identify', 
      'ListMetadataFormats', 
      'ListSets', 
      'GetRecord', 
      'ListIdentifiers', 
      'ListRecords'
    );
    if (!in_array($requestVerb, $requestVerbs)) {
      throw new Exception('OAI-PMH parser error: Invalid request verb.');
    }
    return true;
  }

  /**
   * Validate a URL.
   * @param $url
   * @return unknown_type
   */
  protected static function validateUrl($url) {
    if (!preg_match('/^http/', $url) || !@fopen($url, 'rb')) {
      throw new Exception(
        'OAI-PMH parser error: Could not validate or open provided URL.');
    }
    return true;
  }

  /**
   * Build the schema array containing all the names and namespace URIs of
   * the extended schema in the lib/Schema directory.
   * @return unknown_type
   */
  protected function _setSchemas() {
    foreach (new DirectoryIterator(OAIPMH_SCHEMA_DIRECTORY_PATH) as $item) {
      if ($item->isFile($item)) {
        include_once(OAIPMH_SCHEMA_DIRECTORY_PATH . DIRECTORY_SEPARATOR . $item);
        $schemaName   = substr($item, 0, strpos($item, '.'));
        $className    = 'Schema_' . $schemaName;
        $namespaceUri = call_user_func(array($className, 'getNamespaceUri'));
        $this->_schemas[] = array(
          'schemaName'   => $schemaName, 
          'namespaceUri' => $namespaceUri
        );
      }
    }
  }
}