<?php
/**
 * Copyright 2008 Center for History and New Media
 *
 * This file is part of OAI-PMH Parser Library.
 *
 * OAI-PMH Parser Library is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.

 * OAI-PMH Parser Library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with OAI-PMH Parser Library.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

class OaiPmh_Dom_ListRecords extends OaiPmh_Dom {

  /**
   * Creates response object for the ListRecords response
   * @param $response The OAI-PMH response
   * @return unknown_type
   */
  public function __construct($response) {
    parent::__construct($response);
    $this->_setListRecords();
  }

  /**
   * Parse the the ListRecords response
   * @return unknown_type
   */
  protected function _setListRecords() {
    
    // ListRecords 1
    $listRecordsDomElement = $this->_oaiPmhDomDocument->getElementsByTagNameNS(
      parent::NAMESPACE_URI, 'ListRecords')->item(0);

    // record 1-unbounded
    $recordDomNodeList = $listRecordsDomElement->getElementsByTagNameNS(
      parent::NAMESPACE_URI, 'record');

    foreach ($recordDomNodeList as $recordDomElement) {

      // header 1
      $headerDomElement = $recordDomElement->getElementsByTagNameNS(
        parent::NAMESPACE_URI, 'header')->item(0);

      // identifier 1
      $identifierDomElement = $headerDomElement->getElementsByTagNameNS(
        parent::NAMESPACE_URI, 'identifier')->item(0);

      // datestamp 1
      $datestampDomElement = $headerDomElement->getElementsByTagNameNS(
        parent::NAMESPACE_URI, 'datestamp')->item(0);

      // setSpec 0-unbounded
      $setSpecDomNodeList = $headerDomElement->getElementsByTagNameNS(
        parent::NAMESPACE_URI, 'setSpec');
      $setSpecs = array();
      foreach ($setSpecDomNodeList as $setSpecDomElement) {
        $setSpecs[] = $setSpecDomElement->nodeValue;
      }

      // metadata 0-1
      $metadataDomNodeList = $recordDomElement->getElementsByTagNameNS(
        parent::NAMESPACE_URI, 'metadata');
      $metadata = array();
      foreach ($metadataDomNodeList as $metadataDomElement) {
        $metadata = $this->_getChildNodeByNamespaceUri($metadataDomElement);
      }

      // about 0-unbounded
      $aboutDomNodeList = $recordDomElement->getElementsByTagNameNS(
        parent::NAMESPACE_URI, 'about');
      $abouts = array();
      foreach ($aboutDomNodeList as $aboutDomElement) {
        $abouts[] = $this->_getChildNodeByNamespaceUri($aboutDomElement);
      }

      $this->_oaiPmh['ListRecords']['record'][] = array(
        'header' => array(
            'identifier' => $identifierDomElement->nodeValue, 
            'datestamp' => $datestampDomElement->nodeValue, 
            'setSpec' => $setSpecs
        ),
        'metadata' => $metadata, 
        'about' => $abouts
      );
    }

    // resumptionToken 0-1
    $this->_oaiPmh['ListRecords']['resumptionToken'] = 
        $this->_getResumptionToken($listRecordsDomElement);
  }
}
