<?php
/**
 * Copyright 2008 Center for History and New Media
 *
 * This file is part of OAI-PMH Parser Library.
 *
 * OAI-PMH Parser Library is free software: you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of 
 * the License, or (at your option) any later version.

 * OAI-PMH Parser Library is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with OAI-PMH Parser Library.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

class OaiPmh_Response extends OaiPmh
{
    protected $_requestVerb;
    protected $_requestUrl;
    protected $_response;
    
    /**
     * Creates a response object by OAI verb and full, parameterized URL
     * @param $requestVerb The OAI verb
     * @param $requestUrl Full, parameterized URL
     */
    public function __construct($requestVerb, $requestUrl) {
        OaiPmh::validateRequestVerb($requestVerb);
        OaiPmh::validateUrl($requestUrl);
        $this->_requestVerb = $requestVerb;
        $this->_requestUrl = $requestUrl;
        $this->_setResponse();
    }
    
    /**
     * Get the DOM object containing the parsed OAI-PMH. 
     * Returns an error object if the response is an error.
     * @return unknown_type
     */
    public function getDomObject() {
        if (OaiPmh_Dom::isErrorResponse($this->_response)) {
            $domClass = 'OaiPmh_Dom_Error';
        } else {
            $domClass = 'OaiPmh_Dom_' . $this->_requestVerb;
        }
        return new $domClass($this->_response);
    }
    
    protected function _setResponse() {
      //echo $this->_requestUrl, LN;
      $ch = curl_init("");
      curl_setopt($ch, CURLOPT_HEADER, 0); //1
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
      curl_setopt($ch, CURLOPT_NOPROGRESS, 1);
      curl_setopt($ch, CURLOPT_USERAGENT, 'Omeka-XC OAI harvester');
      curl_setopt($ch, CURLOPT_ENCODING, "");
      curl_setopt($ch, CURLOPT_URL, $this->_requestUrl);
      $this->_response = curl_exec($ch);
      
      //$this->_response = file_get_contents($this->_requestUrl);
    }
}
