<?php
// $Id: XCEntity.php,v 1.2 2009/05/14 16:41:52 mtwesley Exp $

class XCEntity {
  
  /** The type of entity */
  public $xc_type;
  
  /** The ID of entity */
  public $xc_id;
  
  /** The Drupal node type */
  public $node_type = null;
  
  /** The Drupal node ID */
  public $node_id = null;
  
  /** The content of the object as array */
  public $metadata = array();
  
  /** The source data in its original format */
  public $rawdata;
  
  /** The name of the format of the original data */
  public $rawformat;
  
  /** The timestap */
  public $timestamp;
  
  /** The relationships of the entity */
  public $relationships = array();
  
  /** Has the entity been built? */
  public $is_built = false;
  
  /** Has the entity been stored? */
  public $is_stored = false;

  private $locked = false;

  /**
   * Creates a new entity
   * @param $xc_type The type of entity
   * @param $xc_id   The id of entity
   * @return XCEntity The newly created entity
   */
  public function __construct($xc_type = '', $xc_id = '') {
    $this->xc_type = $xc_type;
    $this->xc_id = $xc_id;

    // try to restore from cache
    $_cached = _xc_cache_get('entity instance', _object_to_qualified($this, 'entity instance'));
    if (!empty($_cached)) { 
      $this->node_type     = $_cached->node_type;
      $this->node_id       = $_cached->node_id;
      $this->metadata      = $_cached->metadata;
      $this->rawdata       = $_cached->rawdata;
      $this->rawformat     = $_cached->rawformat;
      $this->timestamp     = $_cached->timestamp;
      $this->relationships = $_cached->relationships;
      $this->is_built      = $_cached->is_built;
      $this->is_stored     = $_cached->is_stored;
    }
    else if (!empty($this->xc_type) && !empty($this->xc_id)) {
      _xc_retrieve($this);
    }
  }

  public function get_node_info() {
    if (empty($this->node_type) || empty($this->node_id)) {
      $entity_table = xc_metadata_entity_get_table($xc_type);
      $sql = "SELECT node_type, node_id
              FROM {$entity_table} WHERE xc_type = '%s' AND xc_id = '%s'";
      $object = db_result(db_query(db_rewrite_sql($sql, "{$entity_table}", ''), $xc_type, $xc_id));
      $this->node_type = $object->node_type;
      $this->node_id = $object->node_id;
    }
  }

  public function get_relationships() { 
    return xc_entity_get_relationships($this);
  }

  public function get_definition() { 
    return xc_metadata_entity_get($this->xc_type);
  }

  public function get_node() {
    return node_load(array('nid' => $this->node_id, 'type' => $this->node_type));
  }

  public function set_relationships() { 
    return xc_entity_set_relationships($this);
  }

  /**
   * Call _xc_build, which constructs entity instance objects from its raw input data 
   * based on the raw import format.
   */
  public function build($params = array()) { 
    _xc_build($this, $this->rawformat, $params);
  }

  /**
   * Call _xc_transform
   * @see _xc_transform
   * @param $format
   * @return unknown_type
   */
  public function transform($format, $params = array()) {
    return _xc_transform($this, $format, $params);
  }

  public function store($locations = array(), $params = array()) {
    _xc_store($this, $locations, $params);
  }

  public function retrieve($locations = array(), $params = array()) {
    _xc_retrieve($this, $locations, $params);
  }

  public function remove($locations = array(), $params = array()) {
    _xc_remove($this, $locations, $params);
  }

  public function alter($params = array()) {
    _xc_alter($this, $params);
  }

  public function validate() { }

  public function clean() { }

  public function fetch_field($field, $delta = 0) {
    return xc_fetch_field($this, $field, $delta);
  }

  public function fetch_attribute($field, $attribute, $delta = 0) {
    return xc_fetch_field($this, $field, $attribute, $delta);
  }

  public function alter_field($field, $delta = 0, $alter = null, $value = null) {
    xc_alter_field($this, $field, $delta, $alter, $value);
  }

  public function alter_attribute($field, $attribute, $delta = 0, $alter = null, $value = null) {
    xc_alter_field($this, $field, $attribute, $delta, $alter, $value);
  }

  public function is_locked() {
    return $this->locked;
  }

}
